import gevent.select
import subprocess
import glob

from ..event import Event
from .input import Input

class File(Input):
    """Tails events from a log file on disk.

    Creates events with the field 'data' set to the line received.

    :param string path: path on the file system to the log file(s), wildcards may
      be used to match multiple files.
    """

    def __init__(self, path):
        super(File, self).__init__()
        self.path = path

    def _run(self):
        # Nice to haves: gevent.subprocess in 1.0

        # This approach is simple but dumb, and has its limitations:
        # - we don't find new files appearing later that match the wildcard
        # - we persist state and pick up file position where we left off
        args = ['tail', '-F']
        paths = glob.glob(self.path)
        if paths:
            # seek to end
            args.append('-n0')
        while not paths:
            gevent.sleep(0.01)
            paths = glob.glob(self.path)

        self.logger.debug('Tailing %s' % ', '.join(paths))
        ps = subprocess.Popen(args + paths,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        while True:
            gevent.select.select([ps.stdout], [], [])
            data = ps.stdout.readline()
            if data:
                if data.endswith('\n'):
                    data = data[0:-1]
                self.logger.debug('Received: %r' % data)
                self.output.put(Event(data=data))



import gevent.socket as socket
import pickle
import struct
import time

from .output import Output

class Graphite(Output):
    """Upload stats data to a graphite server.

    :param string host: graphite server hostname
    :param string port: graphite server port
    """

    def __init__(self, host='localhost', port=2004):
        super(Graphite, self).__init__()
        self.host = host
        self.port = port
        self.connect()
        
    def connect(self):
        self.sock = socket.socket()
        self.sock.connect((self.host, self.port))
        self._metrics = []
        
    def process(self, event):
        metric = event.metric
        timestamp = int(time.mktime(event.timestamp.timetuple()))
        data = []
        for s, value in event.stats.iteritems():
            path = '%s.%s' % (metric, s)
            d = (path, (timestamp, value))
            data.append(d)

        # wire protocol is a 4-byte length, followed by pickled representation,
        # see: http://graphite.readthedocs.org/en/1.0/feeding-carbon.html
        payload = pickle.dumps(data)
        header = struct.pack("!L", len(payload))
        message = header + payload
        self.sock.sendall(message)



import time
import gevent

class ConfigException(Exception):
    pass

def dynamic_class(modname, classname):
    mod = __import__(modname)
    # import returns the top-level module, so recurse to the sub module
    for m in modname.split('.')[1:]:
        mod = getattr(mod, m)
    return getattr(mod, classname)

class BroadcastQueue(list):
    """Queue-like object that broadcasts to all child queues."""

    def put(self, obj):
        for q in self:
            q.put(obj)

    def join(self):
        for q in self:
            q.join()

class Periodic(gevent.Greenlet):
    """Greenlet wrapper that periodically makes a callback."""
    
    def __init__(self, period, callback):
        super(Periodic, self).__init__()
        self.period = period
        self.callback = callback

    def _run(self):
        now = time.time()
        next = now + self.period
        while True:
            while next < now:
                next += self.period
            gevent.sleep(next - now)
            self.callback()
            now = time.time()
